/****************************************************************************

  (c) SYSTEC electronic GmbH, D-07973 Greiz, August-Bebel-Str. 29
      www.systec-electronic.de

  Project:      USB-CANmodul

  Description:  demo using console at Microsoft Visual Studio 6.0

  -------------------------------------------------------------------------

                $RCSfile:$

                $Author:$

                $Revision:$  $Date:$

                $State:$

                Build Environment:
                    Microsoft Visual Studio 6.0

  -------------------------------------------------------------------------

  Revision History:

****************************************************************************/

#include <windows.h>
#include <stdio.h>
//#include <conio.h>
#include "..\include\usbcan32.h"
#include "..\include\ami.h"


/***************************************************************************/
/*                                                                         */
/*                                                                         */
/*          G L O B A L   D E F I N I T I O N S                            */
/*                                                                         */
/*                                                                         */
/***************************************************************************/

//---------------------------------------------------------------------------
// const defines
//---------------------------------------------------------------------------

#define APP_TXMSG_COUNT         100 // number of messages to transmit before shuting down
#define APP_LOOP_WAITTIME       500 // milliseconds

// If APP_RXMSG_DELTATIME is defined then time difference to previous received
// CAN messages will be printed out.
#define APP_RXMSG_DELTATIME


//---------------------------------------------------------------------------
// modul global types
//---------------------------------------------------------------------------


//---------------------------------------------------------------------------
// local vars
//---------------------------------------------------------------------------


//---------------------------------------------------------------------------
// local function prototypes
//---------------------------------------------------------------------------

void PrintCanMsg (tCanMsgStruct* pCanMsg_p, BYTE bChannel_p, DWORD* pdwOldTime_p);


//---------------------------------------------------------------------------
//
// Function:    main()
//
// Description: main function
//
// Parameters:  void
//
// Return:      void
//
// State:       tested r.d. on 12.12.2005
//
//---------------------------------------------------------------------------

#if (_WIN32_WCE == 500)
int WINAPI WinMain (
    HINSTANCE hInstance, 
    HINSTANCE hPrevInstance, 
    LPWSTR lpCmdLine, 
    int nShowCmd)
#else // #elif (_WIN32_WCE == 600)
int wmain (void)
#endif
{

UCANRET             bRet;
tUcanHandle         UcanHandle;
tUcanInitCanParam   CanInitParam;
tCanMsgStruct       RxCanMsg;
tCanMsgStruct       TxCanMsg;
tStatusStruct       UcanStatus;
BYTE                bChannel;
DWORD               dwTxCounter;
DWORD               dwCount;
WORD                wOldCanStatus = USBCAN_CANERR_OK;
DWORD               dwTimeStamp;
DWORD               dwVersion;
tUcanHardwareInfoEx HwInfo;

    printf ("=======================================\n"
            "== USB-CANmodul Console Demo         ==\n"
            "=======================================\n\n");

    printf ("enable debug logfile? (y/n)\n");
    if (getchar () == 'y')
    {
        // set debug mode for USBCAN API
        UcanSetDebugMode (0xE0C00B03L,      // = default Debug-Level (see below)
            _T("\\ConsoleDemo.log"),        // = File Path Name of debug logfile
            0);                             // = flags (1 = append mode, 0 = no append mode)
        //
        // Debug-Level means:
        //      0x00000001L     = function names
        //      0x00000002L     = function parameters
        //      0x00000004L     = function result
        //      0x00000100L     = function UcanReadMsg
        //      0x00000200L     = function UcanWriteMsg
        //      0x00000400L     = function UcanGetStatus
        //      0x00000800L     = all functions for commands
        //      0x00001000L     = all functions for sys-interface
        //      0x00002000L     = all functions for thread-interface
        //      0x00200000L     = function UcanServerThread (not available for WinCE)
        //      0x00400000L     = functions of unvisible window for device notification (not available for WinCE)
        //      0x00800000L     = log of all threads such as data and status thread
        //      0x04000000L     = log of Usbcancp.cpl (not available for WinCE)
        //      0x08000000L     = buffer informations
        //      0x10000000L     = time stamp of several functions
        //      0x20000000L     = dump data
        //      0x40000000L     = log all errors
        //      0x80000000L     = log all standard messages
    }
    printf ("\n");

    // get version of driver
    dwVersion = UcanGetVersionEx (kVerTypeSysDrv);
    printf ("driver   version: V%d.%02d r%d\n",
        USBCAN_MAJOR_VER(dwVersion), USBCAN_MINOR_VER(dwVersion), USBCAN_RELEASE_VER(dwVersion));

    // get version of user library (API)
    dwVersion = UcanGetVersionEx (kVerTypeUserLib);
    printf ("library  version: V%d.%02d r%d\n",
        USBCAN_MAJOR_VER(dwVersion), USBCAN_MINOR_VER(dwVersion), USBCAN_RELEASE_VER(dwVersion));

    // initialize hardware without using callback function
    bRet = UcanInitHardware (&UcanHandle, USBCAN_ANY_MODULE, NULL);
    if (USBCAN_CHECK_ERROR (bRet))
    {
        printf ("ERROR: UcanInitHardware 0x%02X\n", bRet);
        goto Error1;
    }

    // test to receive current time stamp (hardware time stamp)
    bRet = UcanGetModuleTime (UcanHandle, &dwTimeStamp);
    if (USBCAN_CHECK_ERROR (bRet))
    {
        printf ("ERROR: UcanGetModuleTime 0x%02X\n", bRet);
        goto Error2;
    }

    // get hardware info
    memset (&HwInfo, 0, sizeof (HwInfo));
    HwInfo.m_dwSize   = sizeof (HwInfo);
    bRet = UcanGetHardwareInfoEx2 (UcanHandle, &HwInfo, NULL, NULL);
    if (USBCAN_CHECK_ERROR (bRet))
    {
        printf ("ERROR: UcanGetHardwareInfoEx2 0x%02X\n", bRet);
        goto Error2;
    }

    dwVersion = HwInfo.m_dwFwVersionEx;
    printf ("firmware version: V%d.%02d r%d\n",
        USBCAN_MAJOR_VER(dwVersion), USBCAN_MINOR_VER(dwVersion), USBCAN_RELEASE_VER(dwVersion));

    printf ("device    number: %d\n", HwInfo.m_bDeviceNr);
    printf ("serial    number: %d\n", HwInfo.m_dwSerialNr);

    // initialize CAN interface (channel 0)
    memset (&CanInitParam, 0,     sizeof (CanInitParam));
    CanInitParam.m_dwSize       = sizeof (CanInitParam);
    CanInitParam.m_bMode        = kUcanModeNormal;              // since V3.01 kUcanModeTxEcho will be possible
    CanInitParam.m_bBTR0        = HIBYTE (USBCAN_BAUD_1MBit);
    CanInitParam.m_bBTR1        = LOBYTE (USBCAN_BAUD_1MBit);
    CanInitParam.m_bOCR         = USBCAN_OCR_DEFAULT; // only important for GW-001/GW-002
    CanInitParam.m_dwAMR        = USBCAN_CALCULATE_AMR (FALSE, 0x000, 0x7FF, FALSE, TRUE); // receive all
    CanInitParam.m_dwACR        = USBCAN_CALCULATE_ACR (FALSE, 0x000, 0x7FF, FALSE, TRUE);
    CanInitParam.m_dwBaudrate   = USBCAN_BAUDEX_USE_BTR01;
    bRet = UcanInitCanEx2 (UcanHandle, USBCAN_CHANNEL_CH0, &CanInitParam);
    if (USBCAN_CHECK_ERROR (bRet))
    {
        printf ("ERROR: UcanInitCanEx2 0x%02X\n", bRet);
        goto Error2;
    }

    printf ("\n"
            "initialisation successfuly\n"
            "try send %d messages and try to receive... (approx. %d seconds)\n\n",
            APP_TXMSG_COUNT, (APP_TXMSG_COUNT * APP_LOOP_WAITTIME) / 1000);

    // prepare CAN message struct
    TxCanMsg.m_dwID     = 0x391;                // CAN-ID
    TxCanMsg.m_bFF      = USBCAN_MSG_FF_STD;    // CAN2.0A message (no RTR)
    TxCanMsg.m_bDLC     = 0x08;                 // 8 data bytes
    memset (&TxCanMsg.m_bData[0], 0, 8);
    dwTxCounter = 0;

    do
    {
        // set counter to CAN message data
        AmiSetDword (&TxCanMsg.m_bData[0], dwTxCounter);

        // send only one CAN message
        dwCount = 1;  // use an array of tCanMsgStruct if you want to send more than one CAN message
        bRet = UcanWriteCanMsgEx (UcanHandle, USBCAN_CHANNEL_CH0, &TxCanMsg, &dwCount);
        if (USBCAN_CHECK_ERROR (bRet))
        {
            printf ("ERROR: UcanWriteCanMsgEx 0x%02X\n", bRet);
            goto Error3;
        }

        // increment Tx counter
        dwTxCounter++;

        // sometimes read CAN status
        if ((dwTxCounter % 10) == 0)
        {
            // read CAN status
            bRet = UcanGetStatusEx (UcanHandle, USBCAN_CHANNEL_CH0, &UcanStatus);
            if (USBCAN_CHECK_ERROR (bRet))
            {
                printf ("ERROR: UcanGetStatusEx 0x%02X\n", bRet);
                goto Error3;
            }

            // check if CAN status has been changed
            if (UcanStatus.m_wCanStatus != wOldCanStatus)
            {
                printf ("CAN Status changed: 0x%04X\n", UcanStatus.m_wCanStatus);
                wOldCanStatus = UcanStatus.m_wCanStatus;

                // check if bus off status is set
                if (UcanStatus.m_wCanStatus & USBCAN_CANERR_BUSOFF)
                {
                    printf ("BUS OFF reset CAN controler\n");

                    // reset CAN controller
                    UcanResetCanEx (UcanHandle, USBCAN_CHANNEL_CH0, USBCAN_RESET_ONLY_CANCTRL);
                }
            }
        }

        do
        {
            // read only one CAN message
            dwCount  = 1;  // use an array of tCanMsgStruct if you want to read more than one CAN message
            bChannel = USBCAN_CHANNEL_CH0;
            bRet = UcanReadCanMsgEx (UcanHandle, &bChannel, &RxCanMsg, &dwCount);
            if (USBCAN_CHECK_VALID_RXCANMSG (bRet))
            {
                // print out CAN message
                PrintCanMsg (&RxCanMsg, bChannel, &dwTimeStamp);
            }
            else if (USBCAN_CHECK_ERROR (bRet))
            {
                printf ("ERROR: UcanReadCanMsgEx 0x%02X\n", bRet);
                goto Error3;
            }
        }
        while (bRet != USBCAN_WARN_NODATA);

        // wait to prevent much CPU usage
        Sleep (APP_LOOP_WAITTIME);
    }
    while ((dwTxCounter < APP_TXMSG_COUNT));

Error3:
    // deinitialize CAN interface
    UcanDeinitCan (UcanHandle);

Error2:
    // deinitialize hardware
    UcanDeinitHardware (UcanHandle);

Error1:
    printf ("press any key to exit demo\n");
    getchar ();

    return 0;

}


//---------------------------------------------------------------------------
//
// Function:    PrintCanMsg()
//
// Description: prints a CAN message to console window
//
// Parameters:  pCanMsg_p   = CAN message struct
//              bChannel_p  = CAN channel
//
// Return:      void
//
// State:       tested r.d. on 12.12.2005
//
//---------------------------------------------------------------------------

void PrintCanMsg (tCanMsgStruct* pCanMsg_p, BYTE bChannel_p, DWORD* pdwOldTime_p)
{

DWORD dwCount, dwIndex;

    // check if Tx echo was received
    if (pCanMsg_p->m_bFF & USBCAN_MSG_FF_ECHO) // will be available with Version 3.01
    {
        printf ("TX: CH%d ", bChannel_p);
    }
    else
    {
        printf ("RX: CH%d ", bChannel_p);
    }

    // check if CAN2.0B message was received
    if (pCanMsg_p->m_bFF & USBCAN_MSG_FF_EXT)
    {
        printf ("ID=%08X FF=%02X DLC=%d ", pCanMsg_p->m_dwID, pCanMsg_p->m_bFF, pCanMsg_p->m_bDLC);
    }
    else
    {
        printf ("ID=_____%03X FF=%02X DLC=%d ", pCanMsg_p->m_dwID, pCanMsg_p->m_bFF, pCanMsg_p->m_bDLC);
    }

    // check if remote frame war received
    if (pCanMsg_p->m_bFF & USBCAN_MSG_FF_RTR)
    {
        printf ("'Remote Frame            ");
    }
    else
    {
        printf ("'");

        // loop until all data bytes are printed
        dwIndex = 0;
        dwCount = pCanMsg_p->m_bDLC;
        while (dwCount > 0)
        {
            printf ("%02X ", pCanMsg_p->m_bData[dwIndex]);

            // next data byte
            dwIndex++;
            dwCount--;
        }
    }

    // NOTE:
    // Value m_dwTime will be received from Hardware. Thus it is
    // important to know that only 24 bits of the value are valid.
    // It means that the timer value will be reset to zero each
    // 2^24 milliseconds = 4,66 hours. If you need to calculate
    // a time difference you should use the macro USBCAN_CALC_TIMEDIFF().
    #ifdef APP_RXMSG_DELTATIME
    {
        printf ("'time diff=%d\n", USBCAN_CALC_TIMEDIFF (*pdwOldTime_p, pCanMsg_p->m_dwTime));
        *pdwOldTime_p = pCanMsg_p->m_dwTime;
    }
    #else
    {
        printf ("'time=%d\n", pCanMsg_p->m_dwTime);
    }
    #endif

}

