/****************************************************************************

   Definitions for Abstract Memory Interface

   (c) SYS TEC electronic GmbH, Greiz

   06.03.2000  -rs 
               Implementation

   16.09.2002  -as
               To save code space the functions AmiSetByte and AmiGetByte
               are replaced by macros. For targets which assign BYTE by 
               an 16Bit type, the definition of macros must changed to 
               functions.

   23.02.2005  r.d.:
               Functions included for extended data types such as UNSIGNED24,
               UNSIGNED40, ...

   15.02.2008  r.d.:
                - Cleanings for USB-CANmodul under WinCE.
                - AmiSetSwapWord(), AmiSetSwapDword(), AmiGetSwapWord() and AmiGetSwapDword() added

****************************************************************************/

#ifndef _AMI_H_
#define _AMI_H_


//---------------------------------------------------------------------------
//  types
//---------------------------------------------------------------------------


//---------------------------------------------------------------------------
//  Prototypen
//---------------------------------------------------------------------------

#ifdef __cplusplus
    extern "C" {
#endif 

#if defined (_WIN32_WCE)

    //---------------------------------------------------------------------------
    //
    // set functions
    //
    //---------------------------------------------------------------------------

    #define AmiSetByte(pAddr_p,bVal_p)  {*(BYTE FAR*)(pAddr_p) = (bVal_p);}

    void   PUBLIC  AmiSetWord      (void FAR* pAddr_p, WORD wVal_p);
    void   PUBLIC  AmiSetSwapWord  (void FAR* pAddr_p, WORD wVal_p);
    void   PUBLIC  AmiSetDword     (void FAR* pAddr_p, DWORD dwVal_p);
    void   PUBLIC  AmiSetSwapDword (void FAR* pAddr_p, DWORD dwVal_p);


    //---------------------------------------------------------------------------
    //
    // get functions
    //
    //---------------------------------------------------------------------------

    #define AmiGetByte(pAddr_p)  (*(BYTE FAR*)(pAddr_p))

    WORD   PUBLIC  AmiGetWord      (void FAR* pAddr_p);
    WORD   PUBLIC  AmiGetSwapWord  (void FAR* pAddr_p);
    DWORD  PUBLIC  AmiGetDword     (void FAR* pAddr_p);
    DWORD  PUBLIC  AmiGetSwapDword (void FAR* pAddr_p);

#else   // all other platforms which does not need an advanced memory interface

    //---------------------------------------------------------------------------
    //
    // set functions
    //
    //---------------------------------------------------------------------------

    #define AmiSetByte(pAddr_p,bVal_p)   {*(BYTE  FAR*) (pAddr_p) = (bVal_p);}
    #define AmiSetWord(pAddr_p,wVal_p)   {*(WORD  FAR*) (pAddr_p) = (wVal_p);}
    #define AmiSetDword(pAddr_p,dwVal_p) {*(DWORD FAR*) (pAddr_p) = (dwVal_p);}

    void   PUBLIC  AmiSetSwapWord  (void FAR* pAddr_p, WORD wVal_p);
    void   PUBLIC  AmiSetSwapDword (void FAR* pAddr_p, DWORD dwVal_p);

    //---------------------------------------------------------------------------
    //
    // get functions
    //
    //---------------------------------------------------------------------------

    #define AmiGetByte(pAddr_p)  (*(BYTE FAR*)  (pAddr_p))
    #define AmiGetWord(pAddr_p)  (*(WORD FAR*)  (pAddr_p))
    #define AmiGetDword(pAddr_p) (*(DWORD FAR*) (pAddr_p))

    WORD   PUBLIC  AmiGetSwapWord  (void FAR* pAddr_p);
    DWORD  PUBLIC  AmiGetSwapDword (void FAR* pAddr_p);

#endif

#ifdef __cplusplus
    }
#endif 


#endif  // ifndef _AMI_H_

// Please keep an empty line at the end of this file.
